'
'*******************************************************************************
'
' Programm ......... : DAS_1 - Ansteuerung Grafikdisplay
'
' Programminhalt ... : Grundfunktionen zur Darstellung von Grafik/Text/Bildern
'                      - Querformat max. 160x128, Hochformat max. 128x160
'                      Option (Schaltung vorbereitet)
'                      - Einbindung DCF77-Empfang (Funkuhr)
'                      - Helligkeit der Display-Hintergrundbeleuchtung (PWM)
'
' Code-Anpassung ... : Ingolf Bauer (ingolf.bauer@nexgo.de)
'  Quellenangabe ... :  Copyright by H. Kipnik 2012
'
'                      Programm fr Display 1.8 Zoll 128x160 Pixel mit
'                               Controler ST7735 oder ILI9163
'                      Bascom IDE 2.0.7.7
'                      Version: 13012014
'                      Date:    13.01.2014
'
'                      Diplay:
'                      - $include "TFTDriver\ST7735_declarations.inc"
'                      - $include "TFTDriver\ST7735_functions.inc"
'                      SDCard:
'                      - $include "Avr-Dos\Config_MMCSD_HC.bas"  (Anpassung CS)
'                      - $include "Avr-Dos\Config_AVR-DOS.bas"
'                      - $include "Avr-Dos\SD_card_init.inc"
'                      Font:
'                      - $include "Font\font8x8.font"
'                      - $include "Font\font12x16.font"
'                      Font (optional):
'                      - $include "Font\font6x10.font"
'                      - $include "Font\font8x12.font"
'                      - $include "Font\font10x16.font"
'
' Version .......... : 0.2.91
' Datum ............ : 28.04.2021
'
' Hinweise
' Label A_...          Die mit dem Prfix "A_" versehenen Labels dienen nur dem
'                      schnelleren Auffinden der Zeile im Programmcode.
' Label L/M_...        Sprungmarken des Programms
'
'-------------------------------------------------------------------------------
'
'Simulator-Einstellmglichkeit
'-----------------------------
'$sim                                             'Wartezeit aus
'
' Achtung!
' Diese Zeile ist unbedingt auszukommentieren, bevor der AVR programmiert wird,
' da sonst der AVR nicht korrekt arbeitet.
' Die Aktivierung ist nur fr den Simulator vorgesehen.
'
'-------------------------------------------------------------------------------
'
'-------------------------------------------------------------------------------
'00. === I N F O R M A T I O N =================================================
'-------------------------------------------------------------------------------
'
A_010_bedienung:                                  'Editmarker: Bedienung
'
'Bedienung
' %
'
'Anzeige
' %
'
'Display
' 01. Andere Bezeichnungen fr den Anschluss "A0" sind "DC" und "RS".
' 02. Pin "LED" muss mit Vcc verbunden werden. Es gibt den Hinweis, bei
'     - Vcc 3,3 V  5-10 Ohm
'     - Vcc 5,0 V 30-50 Ohm
'     vorzuschalten.
'     Ohne einer Beleuchtung ist auf dem Display nichts zu erkennen.
'
'-------------------------------------------------------------------------------
A_020_historie:                                   'Editmarker: Historie
'
'Versionsbersicht
'-----------------
' Label "A_163_version": LCD-Anzeige der Versionsnummer aktualisiern
'
'
'
'
'
'
'
'
'
'
'V 0.2.91  28.04.2021 - Erweiterung . : - Steuerung LED und Ton (Morsecode)
'                     - AVR ......... : - 77%
'V 0.2.93  26.04.2021 - Information . : - Kommentierung Programmcode gendert
'                                       - Aktivierung SDCard
'                     - AVR ......... : - 74%
'V 0.2.97  17.04.2021 - Information . : - Signal "CS" der SDCard engepasst
'V 0.2.99  17.04.2021 - Information . : - Umstellung auf Modul-PCB
'                     - AVR ......... : - 53%
'V 0.1.97  05.02.2021 - Information . : - auf Quelle H. Kipnik (V12) umgestellt,
'                                         da Anzeige prizipiell vorhanden, aber
'                                         Format- und Anzeigefehler
'                                       - Anzeigefehler entstehen durch Steuer-
'                                         signale P5V
'                                         (Levelshifter erforderlich)
'                     - AVR ......... : - 53%
'V 0.1.99  25.01.2021 - Information . : - Beginn der Softwareanpassung
'                     - AVR ......... : - 73%
'
'-------------------------------------------------------------------------------
'
'Informationen zur Software
'--------------------------
' Inhalt                                                                | Zeile
'-------------------------------------------------------------------------------
' 01. Allgemeine Informationen ........................................ |  149
' 02. Abkrzungsbersicht ............................................. |  161
' 03. Compilereinstellungen ........................................... |  166
' 04. Funktionsbeschreibung ........................................... |  189
' 05. Interrupt ....................................................... |  195
' 06. Einstellung der Fuse- und/oder Lock-Bits ........................ |  199
' 07. Anschluschema fr den AVR ATmega328P ........................... |  246
' 08. I/O-Ports ....................................................... |  287
'      Abkrzungen fr die Signalbezeichnung .......................... |  289
' 09. EEprom-Belegung ................................................. |  371
' 10. Allgemein ....................................................... |  381
' 11. Schaltkreis ..................................................... |  386
'      Watchdog ....................................................... |  407
'      Schnittstelle .................................................. |  412
'      Compilereinstellung ............................................ |  417
' 12. Fuse- und Lockbits .............................................. |  425
' 13. Deklaration ..................................................... |  443
' 14. Konfiguration ................................................... |  476
'      I/O-Port ....................................................... |  479
' 15. Alias ........................................................... |  506
' 16. Konstanten ...................................................... |  529
' 20. Programm ........................................................ |  546
'      Initialisierung ................................................ |  551
'      bernahme Programmcode ......................................... |  558
'      Bedingte Kompilierung .......................................... |  563
'      Display initialisieren ......................................... |  579
'      SDCard initialisieren .......................................... |  584
' 40. Programm ........................................................ |  871
'      Tonausgabe ..................................................... |  873
' 70. Include ......................................................... |  911
' 80. Datenbereich .................................................... |  930
'-------------------------------------------------------------------------------
'01. Allgemeine Informationen
'----------------------------
'Taktfrequenz
' Fr die Erzeugung der Taktfrequenz wird ein Quarz (16,0 MHz) verwendet.
'
'Programmierung
' Die Programmierung des MCs kann extern oder ber die vorhandene Schnittstelle
' erfolgen.
' Zur Programmierung wurde das Modul "USBasp" verwendet, welches ber eine
' USB-Schnittstelle angeschlossen wird. Es kann direkt ber eine, in BASCOM
' integrierte Einstellung angesprochen werden.
'
'02. Abkrzungsbersicht
'-----------------------
' HGB ... : Hintergrundbeleuchtung des Displays
' PWM ... : Pulsweitenmodulation
'
'03. Compilereinstellungen
'-------------------------
' Hinweise fr ATmega8/ATmega16 mit 1024 Byte SRAM
' Quelle:
' //halvar.at/elektronik/kleiner_bascom_avr_kurs/speicher_hwstack_swstack_frame/
'
' HW-Stack .. - Empfehlung: 40 Byte bei INT, sonst 10 Byte ausreichend
'  32 Byte fr INT + je GOSUB 2 Byte, wenn bei INT ein GOSUB erfolgt
'  (bei Verschachtelung ... + n x 2 Byte)
'
' SW-Stack .. - Empfehlung: 32 Byte (recht komplexe Aufrufe werden abgedeckt)
'   2 Byte fr jede bergebene Variable einer GOSUB-Routine + 2 Byte fr jede
'   LOCAL-Variable innerhalb einer GOSUB
'   (bei Verschachtelung ... n x 2 Byte)
'
' Frame-Space - Empfehlung: 60 Byte sollten alles abdecken
'                           (bei wenig Speicher verkleinern)
'  Beispiel:
'   2 x BYTE        =  2 Byte (erster und zweiter Parameter)
'   1 x WORD        =  2 Byte (dritte Parameter)
'   1 x STRING * 10 = 11 Byte (inkl. Abschlussbyte; vierter Parameter)
'   3 x BYTE        =  3 Byte (lokale Variablen)
'
'04. Funktionsbeschreibung
'-------------------------
' Allgemeines
' - - - - - -
' %
'
'05. Interrupt
'-------------
' siehe Hinweise zu: Timer, Watchdog, ...
'
'06. Einstellung der Fuse- und/oder Lock-Bits
'--------------------------------------------
' - Fuse-Einstellungen fr Brenner USBasp (Bascom)
'   Die aktuellen Einstellungen befinden sich im Abschnitt "A_120_fuse".
'
' - Fuse-bersicht
'   Die angegebenen HEX-Werte knnen direkt in den compilierten Programmcode
'   eingetragen werden, falls dies untersttzt wird.
'   Beim USBasp unter Bascom ist dies nicht mglich.
'
' Bedeutung der Fuse/Lock-Bit-Einstellungen
' Name               ATmega328
'   Calibration 0    9B  (ohne Bedeutung)
' * Lockbits         FF
'    Lockbit 65       11: No restrictions for SPM or LPM accessing ...
'    Lockbit 43       11: No restrictions for SPM or LPM accessing ...
'    Lockbit 21       11: No memory lock features enabled for parallel ...
' * Fusebits         FA
'    Fusebit C         1: Divide Clock by 8 disabled
'    Fusebit B         1: Clock Output disabled
'    Fusebit KLA987   111010: Ext. Crystal Osc.
' * Fusebits Hight   D1
'    Fusebit High M    1: Pin PC6 is Reset
'    Fusebit High J    1: debugWIRE disabled
'    Fusebit High I    0: SPI enabled
'    Fusebit High H    1: WDT enabled by WDTCR
'    Fusebit High G    0: Preserve EEPROM when chip erase
'    Fusebit High FE  00: Bootsize 2048 words
'    Fusebit High D    1: Select Reset vector (0000)
' * Fusebit Extended FF
'    Fusebit RSQ     111: Brown out disabled
'
'/// Achtung ///
'- Fusebit High M
'  Bei dem Wert 0 ist mit einfachen Brennern kein Programmieren mehr mglich,
'  da dafr die Reset-Funktion erforderlich ist.
'
' /// Achtung ///
'Gltig fr MC mit folgenden Einstellmglichkeiten:
' - CKSEL
'   Wird CKSEL3= 0 gesetzt, arbeitet der MC nur noch mit einem externen RC-
'   Generator [f= 1/(3RC); C>= 22p].
'
' - RSTDISBL
'   Wird RSTDISBL= 1 gesetzt, ist mit einfachen Brennern kein Programmieren
'   mehr mglich, da dafr die Reset-Funktion unbedingt erforderlich ist.
'
'07. Anschluschema fr den AVR ATmega328P
'-----------------------------------------
A_070_pin:                                        'Editmarker: Pin
'
'/// Achtung ///
'Die Pinbelegung bezieht sich auf einen ATmega328(A) im TQFP32-Gehuse.
'
'                   -------------------------------------
' [0]  LED_rt <--- | 1 [PD3, INT1]         [INT0, PD2] 32| <--- O_2       [~]
' [0]  LED_gn <--- | 2 [PD4, T0]            [TXD, PD1] 31| <--- O_1       [~]
' [=]     GND ==== | 3 [GND]                [RXD, PD0] 30| <--- DCF       [^]
' [=]   P5,0V ==== | 4 [VCC]             [/RESET, PC6] 29| ---> /Reset    [$]
' [=]     GND ==== | 5 [GND]                [SCL, PC5] 28| #### SCL       [#]
' [=]     P5V ==== | 6 [VCC]                [SDA, PC4] 27| #### SDA       [#]
' [+]      QZ ++++ | 7 [PB6, XTAL1]        [ADC3, PC3] 26| ---- E/A_R7    [-]
' [+]      QZ ++++ | 8 [PB7, XTAL2]        [ADC2, PC2] 25| ---- E/A_R6    [-]
'                   -------------------------------------
' [0]  LED_bl <--- | 9 [PD5, T1]           [ADC1, PC1] 24| ---> SD_CS     [1]
' [0]     Ton <--- |10 [PD6, AIN0]         [ADC0, PC0] 23| ---> D_LED     [0]
' [0]    E-R1 ---> |11 [PD7, AIN1]              [ADC7] 22|                [.]
' [0]    D_RT <--- |12 [PB0, ICP1]              [AGND] 21| ==== GND       [=]
' [0]    D_A0 <--- |13 [PB1, OC1A]              [AREF] 20| -||- GND       [=]
' [0]    D_CS <--- |14 [PB2, /SS]               [ADC6] 19|                [.]
' [$]    MOSI <--- |15 [PB3, MOSI]              [AVCC] 18| ==== P5,0V     [=]
' [$]    MISO ---> |16 [PB4, MISO]          [SCK, PB5] 17| <--- SCK       [$]
'                   -------------------------------------
'
'===============================================================================
'
' Standard-Einstellungen nach dem Programmstart
'  [^]: Impuls              [~]: unbestimmt          [&]: PWM
'  [0]: aus (GND)           [1]: ein (Vcc)           [%]: analog
'  [$]: SPI-Bus             [W]: 1-Drahtbus          [#]: I2C-Bus
'  [=]: Stromversorgung     [+]: Quarz               []: nicht verwendbar
'  [.]: nicht angeschlossen (Reservekanal)           [-]: nicht relevant
'
' Datenrichtung ..... : ---- / ---> / <--- / <-->
' Stromversorgung ... : ====
' Quarz ............. : ++++
' Kondensator ....... : -||-
'
'08. I/O-Ports
'-------------
'Abkrzungen fr die Signalbezeichnung
'-------------------------------------
'Prfix
'~~~~~~
'   D_    : Display
' LED_    : Led      (Option: rt: rot, gn: grn, bl: blau)
'  SD_    : SDCard
'
'Suffix
'~~~~~~
'  _CS    : Chip Select
'  _A0    : Richtungssteuerung (oder _DC)
'  _RT    : Reset
'
'Allgemein
'~~~~~~~~~
' Px,yV   : Stromversorgung P5,0V / P3,3V
' GND     : Stromversorgung Masse
'
' MISO    : Programmieranschlu
' MOSI    : Programmieranschlu
' SCK     : Programmieranschlu
'
' QZ      : Anschlu Quarz
'
' DA      : digitaler Ausgang
' DE      : digitaler Eingang
' DX      : (frei konfigurierbar)
' DCF     : Funkempfngersignal
' I2C     : Busprotokoll
' PG      : Programmierung
' SV      : Stromversorgung
' SPI     : Busprotokoll (Programmierung, Display- und SDCardsteurung)
' TWI     : Busprotokoll (optionale Erweiterung)
' USI     : Busprotokoll (universelle Schnittstelle)
'
' %         Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' -   : SV  P5,0V ...... | ............. | Stromversorgung .............. |  4
' -   : SV  P5,0V ...... | ............. | Stromversorgung .............. |  6
' -   : SV  GND ........ | ............. | SV-Masse ..................... |  3
' -   : SV  GND ........ | ............. | SV-Masse ..................... |  5
' -   : SV  P5,0V ...... | ............. | Stromversorgung (analog) ..... | 18
' -   : SV  GND ........ | ............. | SV-Masse ...... (analog) ..... | 21
' -   : SV  Referenz ... | ............. | Referenzeingangsspannung ..... | 20
' -   : PG  Reset ...... | /Reset ...... | PG/ Rcksetzung des MCs ...... | 29
' -   : QZ  Systemtakt . | ............. | Frequenznormal mit Quarz ..... |  7
' -   : QZ  Systemtakt . | ............. | Frequenznormal mit Quarz ..... |  8
'
'Port B     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DA  Reset ...... | D_RESET ..... | Display Steuerung ............ | 12
' - .1: DA  Dat./Comm. . | D_DC ........ | Display Steuerung ............ | 13
' - .2: DA  Chipselect . | D_CS ........ | Display Steuerung ............ | 14
' - .3: PG  SPI ........ | MOSI ........ | Programmierung/Datentransfer . | 15
' - .4: PG  SPI ........ | MISO ........ | Programmierung/Datentransfer . | 16
' - .5: PG  SPI ........ | SCK ......... | Programmierung/Datentransfer . | 17
' - nc: ..  (QZ) ....... | ............. | (siehe oben) ................. |  7
' - nc: ..  (QZ) ....... | ............. | (siehe oben) ................. |  8
'
'Port C     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DA  HGB ........ | D_LED ....... | Display Hintergrundbeleuchtung | 23
' - .1: DA  Chipselect . | SD_CS ....... | SDCard Steurung .............. | 24
' - .2: DX  ............ | ............. | (frei konfigurierbar) ........ | 25
' - .3: DX  ............ | ............. | (frei konfigurierbar) ........ | 26
' - .4: BP  TWI ........ | ............. | (frei konfigurierbar) ........ | 27
' - .5: BP  TWI ........ | ............. | (frei konfigurierbar) ........ | 28
' - .6: ..  ............ | ............. | (analoge Anwendungen) ........ | 19
' - .7: ..  ............ | ............. | (analoge Anwendungen) ........ | 22
'
'Port D     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DE Zeitsignal... | DCF ......... | Funkuhr-Signal ............... | 30
' - .1: DE Konfiguration | O_1 ......... | Option 1...................... | 31
' - .2: DE Konfiguration | O_2 ......... | Option 2 ..................... | 32
' - .3: DA RGB-LED ..... | LED_rt ...... | LED-Ansteuerung rot .......... |  1
' - .4: DA RGB-LED ..... | LED_gn ...... | LED-Ansteuerung grn ......... |  2
' - .5: DA RGB-LED ..... | LED_bl ...... | LED-Ansteuerung blau ......... |  9
' - .6: DA aku. Signal . | Ton ......... | Signalausgabe ................ | 10
' - .7: DE ............. | ............. | (frei konfigurierbar) ........ | 11
'
'09. EEprom-Belegung
'-------------------
A_090_eeprom:                                     'Editmarker Eeprom
'
'Adresse   | Inhalt
' Dez  Hex |
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' 001  01  | %
'
'-------------------------------------------------------------------------------
'10. === A L L G E M E I N =====================================================
'-------------------------------------------------------------------------------
'
A_110_schaltkreis:                                'Editmarker Schaltkreis
'
'11. Schaltkreis
'---------------
   $regfile = "m328pdef.dat"                      'Prozessor ...... : ATmega328P
   $crystal = 16000000                            'Quarzfrequenz .. : 16,0 MHz

'Hinweis zur Datei "~def.dat"
'Funktion "Picopower"
'~~~~~~~~~~~~~~~~~~~
'  ATmega328 : ohne
'  ATmega328P: mit
'Gehuse
'~~~~~~~
'  ATmega328P-AU: PDIP 28
'  ATmega328P-MU: TQFP 32
'  ATmega328P-PU: QFN  32
'  MU/PU haben zwei zustzliche analoge Eingnge, die nicht als digitale I/O
'  verwendet werden knnen.
'
'  Die Anschlsse z. B. eines ATmega8 sind zum jeweiligen Gehusetyp pinkom-
'  patibel.

'Watchdog
'--------
' nicht verwendet
'   Config Watchdog = 2048                         'Reset nach Timeout 2048 ms

'Schnittstelle
'-------------
'ggf. fr eine Ausgabe im Simulator-Feld UART
   $baud = 57600                                  'Baudrate ........ : 57600
                                                  '9600/19200/38400/57600/115200
'Compilereinstellung
'-------------------                              'bisher:
   $hwstack = 50                                  '--
   $swstack = 150                                 '--
   $framesize = 150                               '--

A_120_fuse:                                       'Editmarker: Fuse
'
'12. Fuse- und Lockbits
'----------------------
'USBasp: Werte generiert mit "Write PRG"
' $prog &HFF, &HFA, &HD1, &HFF
'
' FF: 11 11 11
' FA: 1 1 111010
' D1: 1 1 0 1 0 00 1
' FF: 111
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' Archiv
' $PROG &HFF, &HFF, &HD1, &H00
'
'-------------------------------------------------------------------------------
'
A_000_deklaration:                                'Editmarker: Deklaration
A_130_deklaration:                                'Editmarker: Deklaration
'
'13. Deklaration
'---------------
'
'Unterprogramme (Call-Routinen)
' Fehlercode (Tonfolge)

'Array
' %
'Bibliothek
' %
'Flag
' %
'Merker
' Hilfsmerker
   Dim Hm_mc As Byte                              'Code Morsezeichen

'String
   Dim Temp_str As String * 15                    'nur bei SDCard erforderlich
' Morsezeichen (Signalton)
   Dim Hs_mor As String * 5                       'Morsezeichen
' Hilfsstring
   Dim Hs01 As String * 1                         'Hilfsstring 01
'
'Timer
' %
'Variable
' %
'Zhler
' Hilfszhler
   Dim Hz_mz As Byte                              'Morsezeichen
'
'-------------------------------------------------------------------------------
'
'14. Konfiguration
'-----------------
A_141_port:                                       'Editmarker: Port
'I/O-Port
'--------
'Abkrzungen
' Datenrichtung   Pullup-Widerstand
'  Ein: Eingang    Akt: aktiv
'  Aus: Ausgang    Pas: passiv
'
'=Port B
'-Datenrichtung (Ein [0], Aus [1])
   Ddrb = &B0000_0111                             'PB3-7 Ein; PB0-2 Aus
'-Pullup-Widerstand (Pas [0], Akt [1])
   Portb = &B0000_0000                            'PB.-. Akt; PB0-7 Pas
'=Port C
'-Datenrichtung (Ein [0], Aus [1])
   Ddrc = &B0000_0011                             'PC2-7 Ein; PC0-1 Aus
'-Pullup-Widerstand (Pas [0], Akt [1])
   Portc = &B0000_0010                            'PC1-. Akt; PC0,2-7 Pas
'=Port D
'-Datenrichtung (Ein [0], Aus [1])
   Ddrd = &B0111_1000                             'PD2-5,7 Ein; PD3-6 Aus
'-Pullup-Widerstand (Pas [0], Akt [1])
   Portd = &B0000_0000                            'PD.-. Akt; PD0-7 Pas
'
'-------------------------------------------------------------------------------
'
A_150_alias:                                      'Editmarker: Alias
'
'15. Alias
'---------
'Port B
'Ausgang (digital)
'~~~~~~~~~~~~~~~~~
   Lcd_reset Alias Portb.0                        'Display: Reset
   Lcd_dc Alias Portb.1                           'Display: A0(D/C)
   Lcd_cs Alias Portb.2                           'Display: CS
   Lcd_hgb Alias Portc.0                          'Display: HGB

'Port D
'Ausgang (digital)
'~~~~~~~~~~~~~~~~~
   Led_rt Alias Portd.3                           'RGB-Led: rot
   Led_gn Alias Portd.4                           'RGB-Led: grn
   Led_bl Alias Portd.5                           'RGB-Led: blau

   Ton Alias Portd.6                              'signalton
'
'-------------------------------------------------------------------------------
'
A_160_konstante:                                  'Editmarker: Konstante
'
'16. Konstanten
'--------------
A_161_flag:                                       'Editmarker: Flag
'Benutzung einer Speicherkarte
' Bei Benutzung steigt der Programm-Speicherbedarf.
   Const Sdcard = 1                               'Ja: 1, Nein: 0
'Ausrichtung auf dem Display
   Const Modus = 1                                'Portrait: 0, Landscape: 1
'Chipauswahl der Grafikansteuerung
   Const Driver = 0                               'ST7735: 0, (ILI9163: 1)
'Typ- der RGB-Ansteuerung (nur ST7735)
' Beim Grafikchip gibt es unterschiedliche Verfahren der RGB-Ansteuerung.
' Den Unterschied erkennt man am Farbrand der Display-Schutzfolie.
' (schwarz bzw. grn, sowie rot)
   Const Disp_typ = 0                             'sw/gn: 0, rt: 1
'
'-------------------------------------------------------------------------------
'20. === P R O G R A M M =======================================================
'-------------------------------------------------------------------------------
'
A_210_init:                                       'Editmarker: Initialisierung
'
'Initialisierung
'---------------
   Set Lcd_reset                                  'Set Reset
   Set Lcd_cs                                     'Set Chipselect
   Set Lcd_dc                                     'Set Datenrichtung
   Set Lcd_hgb                                    'Set HGB
'
'bernahme Programmcode
'----------------------
'Grafiktreiber (ST7735)
   $include "TFTDriver\ST7735_declarations.inc"   'Include Deklaration

'Bedingte Kompilierung
'---------------------
' SDCard: ja
   #if Sdcard = 1
      $include "Avr-Dos\Config_MMCSD_HC.bas"      'angepasste Quelle fr SD_CS
      $include "Avr-Dos\Config_AVR-DOS.bas"
   #endif

' SDCard: nein (Initialisierung SPI-Schnittstelle)
   #if Sdcard = 0
      Config Spi = Hard , Interrupt = Off , Data Order = Msb , Master = Yes , _
             Polarity = High , Phase = 1 , Clockrate = 4 , Noss = 1
      Spsr.0 = 1
      Spiinit
   #endif

'Display initialisieren
'----------------------
   Call Lcd_init()
   Lcd_clear Black

'SDCard initialisieren
'---------------------
   #if Sdcard = 1
      Call Init_sd_card()
      Temp_str = "SD Card= " + Str(sd_ok)
      Lcd_text Temp_str , 1 , 1 , 1 , Yellow , Black
'Anzeigecode
'  0 Error
'  1 OK

      Temp_str = "SD Card Typ= " + Str(mmcsd_cardtype)
      Lcd_text Temp_str , 1 , 20 , 1 , Yellow , Black
'Anzeigecode
'  0 can't init the Card
'  1 MMC
'  2 SDSC Spec. 1.x
'  4 SDSC Spec. 2.0 or later
' 12 SDHC Spec. 2.0 or later
      Wait 3
      Lcd_clear Black                             'Display: sw (lschen)

A_163_version:
'Anzeige Version und Logo
     Call Lcd_pic(0 , 0 , 160 , 128 , "bild00.bin")       'Startlogo
     Call Lcd_text( "Demo-Modus" , 18 , 20 , 2 , Blue , Transparent)
     Call Lcd_text( "Version 0.2.91" , 22 , 40 , 1 , Blue , Transparent)
     Call Lcd_text( "05/2021" , 73 , 114 , 1 , Black , Transparent)
     Wait 10
   #endif

'Programmtest
' Anzeige LED-RGB
   Set Led_rt : Waitms 500 : Reset Led_rt         'LED rt: E -> A
   Set Led_gn : Waitms 500 : Reset Led_gn         'LED gn: E -> A
   Set Led_bl : Waitms 500 : Reset Led_bl         'LED bl: E -> A
' Signalton
   Hs_mor = "o" : Gosub Up_mor                    'Tonausgabe "o" (---)

A_220_programm:                                   'Editmarker: Programm
'
Do
'Darstellung (Hochformat)
'========================
   #if Modus = 0                                  'Modus Portrait

'Darstellung Farben
'~~~~~~~~~~~~~~~~~~
      Call Lcd_clear(black)                       'Display: sw (lschen)
      Wait 3
      Call Lcd_clear(red)                         'Display: rt
      Wait 3
      Call Lcd_clear(black)                       'Display: sw (lschen)
      Wait 3
      Call Lcd_clear(yellow)                      'Display: ge
      Wait 3
      Call Lcd_clear(black)                       'Display: sw (lschen)
      Wait 3

'Darstellung Text
'~~~~~~~~~~~~~~~~
      Call Lcd_text( "Font 8x8" , 5 , 40 , 1 , _
                      Yellow , Transparent)       'Text: ge
      Call Lcd_text( "Font 12x16" , 1 , 80 , 1 , _
                        Blue , Transparent)       'Text: bl
'      Call Lcd_text( "Font 6x10" , 5 , 20 , 2 , _
'                         Red , Transparent)      'Text: rt
'      Call Lcd_text( "Font 8x12" , 5 , 60 , 1 , _
'                       Green , Transparent)      'Text: gn
'      Call Lcd_text( "Font 10x16" , 1 , 110 , 1 , _
'                      Yellow , Transparent)      'Text: ge
      Wait 3

'Darstellung Linie
'~~~~~~~~~~~~~~~~~
      Call Lcd_clear(black)                       'Display: sw (lschen)
      Call Lcd_text( "Line" , 40 , 2 , 2 , _
                      Yellow , Black)             'Text: ge; Line
      Call Lcd_line(1 , 1 , 127 , 1 , 1 , Yellow)
      Call Lcd_line(1 , 1 , 1 , 159 , 1 , Yellow)
      Call Lcd_line(127 , 1 , 127 , 159 , 1 , Yellow)
      Call Lcd_line(1 , 159 , 127 , 159 , 1 , Yellow)
      Call Lcd_line(1 , 1 , 127 , 159 , 2 , Yellow)
      Call Lcd_line(127 , 1 , 1 , 159 , 2 , Yellow)
      Wait 3

'Darstellung Kreis, gefllt
'~~~~~~~~~~~~~~~~~~~~~~~~~~
      Call Lcd_clear(black)                       'Display: sw (lschen)
      Call Lcd_text( "Circle fill" , 18 , 2 , 2 , _
                      Yellow , Black)             'Text: ge; Circle fill
      Call Lcd_fill_circle(64 , 80 , 40 , Yellow)
      Call Lcd_fill_circle(64 , 80 , 30 , Red)
      Call Lcd_fill_circle(64 , 80 , 20 , Green)
      Call Lcd_fill_circle(64 , 80 , 10 , Blue)
      Call Lcd_fill_circle(64 , 80 , 5 , Yellow)
      Wait 3

'Darstellung Kreis
'~~~~~~~~~~~~~~~~~
      Call Lcd_clear(black)                       'Display: sw (lschen)
      Call Lcd_text( "Circle" , 38 , 2 , 2 , _
                      Yellow , Black)             'Text: ge; Circle
      Call Lcd_circle(64 , 80 , 40 , Red)
      Call Lcd_circle(64 , 80 , 30 , Yellow)
      Call Lcd_circle(64 , 80 , 20 , Green)
      Call Lcd_circle(64 , 80 , 10 , Blue)
      Call Lcd_circle(64 , 80 , 5 , Yellow)
      Wait 3

'Darstellung Rechteck, gefllt
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      Call Lcd_clear(black)                       'Display: sw (lschen)
      Call Lcd_text( "Box Fill" , 10 , 2 , 2 , _
                      Yellow , Black)             'Text: ge; Box fill
      Call Lcd_box(14 , 40 , 100 , 100 , 1 , Blue , Red)
      Call Lcd_box(24 , 50 , 80 , 80 , 1 , Red , Red)
      Call Lcd_box(34 , 60 , 60 , 60 , 1 , Yellow , Red)
      Call Lcd_box(44 , 70 , 40 , 40 , 1 , Green , Red)
      Call Lcd_box(54 , 80 , 20 , 20 , 1 , Brown , Red)
      Wait 3

'Darstellung Rechteck
'~~~~~~~~~~~~~~~~~~~~
      Call Lcd_clear(black)                       'Display: sw (lschen)
      Call Lcd_text( "Box " , 5 , 2 , 2 , _
                      Yellow , Black)             'Text: ge; Box
      Call Lcd_box(14 , 40 , 100 , 100 , 2 , Blue , Red)
      Call Lcd_box(24 , 50 , 80 , 80 , 2 , Red , Green)
      Call Lcd_box(34 , 60 , 60 , 60 , 2 , Yellow , Blue)
      Call Lcd_box(44 , 70 , 40 , 40 , 2 , Green , Red)
      Call Lcd_box(54 , 80 , 20 , 20 , 2 , Brown , White)
      Wait 3

      Call Lcd_clear(black)                       'Display: sw (lschen)
      Call Lcd_text( "Box" , 10 , 2 , 2 , _
                      Yellow , Black)             'Text: ge; Box
      Call Lcd_box(14 , 40 , 100 , 100 , 3 , Blue , Red)
      Call Lcd_box(24 , 50 , 80 , 80 , 3 , Red , Green)
      Call Lcd_box(34 , 60 , 60 , 60 , 3 , Yellow , Blue)
      Call Lcd_box(44 , 70 , 40 , 40 , 3 , Green , Red)
      Call Lcd_box(54 , 80 , 20 , 20 , 3 , Brown , White)
      Wait 3

'Darstellung Bilder (bei aktiver Speicherkarte)
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'Fr eine Anzeige sind die entsprechenden Bilddateien auf der SDCard abzulegen.
      #if Sdcard = 1
         Call Lcd_clear(black)
         Call Lcd_pic(60 , 60 , 50 , 50 , "hv01.bin")
         Wait 3
         Call Lcd_pic(0 , 0 , 128 , 160 , "hv02.bin")
         Call Lcd_text( "Transparent" , 10 , 50 , 2 , _
                         Red , Transparent)
         Wait 3
         Call Lcd_pic(0 , 0 , 128 , 160 , "hv03.bin")
         Wait 3
         Call Lcd_pic(0 , 0 , 128 , 160 , "hv04.bin")
         Wait 3
      #endif

      Call Lcd_clear(red)                         'Display: rt (lschen)
      Wait 3
   #endif

'Darstellung (Querformat)
'========================
   #if Modus = 1                                  'Modus Landscape

'Darstellung Farben
'~~~~~~~~~~~~~~~~~~
      Call Lcd_clear(black)                       'Display: sw (lschen)
      Wait 3
      Call Lcd_clear(yellow)                      'Display: ge
      Waitms 500
      Call Lcd_clear(red)                         'Display: rt
      Waitms 500
      Call Lcd_clear(green)                       'Display: gn
      Waitms 500

'Darstellung Text
'~~~~~~~~~~~~~~~~
      Call Lcd_clear(black)                       'Display: sw (lschen)

      Call Lcd_text( "Font 8x8" , 5 , 40 , 1 , _
                      Yellow , Transparent)       'Text: ge
      Call Lcd_text( "Font 12x16" , 5 , 80 , 2 , _
                      Blue , Transparent)         'Text: bl
      Wait 3

'Darstellung Linie
'~~~~~~~~~~~~~~~~~
      Call Lcd_clear(black)                       'Display: sw (lschen)
      Call Lcd_text( "Line" , 60 , 5 , 2 , _
                      Yellow , Black)             'Text: ge; Line
      Call Lcd_line(0 , 0 , 159 , 0 , 1 , White)
      Call Lcd_line(0 , 127 , 159 , 127 , 1 , White)
      Call Lcd_line(0 , 0 , 0 , 127 , 1 , White)
      Call Lcd_line(159 , 0 , 159 , 127 , 1 , White)
      Call Lcd_line(1 , 1 , 158 , 126 , 2 , Red)
      Call Lcd_line(1 , 126 , 158 , 1 , 2 , Yellow)
      Wait 3

'Darstellung Kreis, gefllt
'~~~~~~~~~~~~~~~~~~~~~~~~~~
      Call Lcd_clear(white)                       'Display: ws
      Call Lcd_fill_circle(22 , 30 , 15 , Green)
      Call Lcd_fill_circle(138 , 30 , 15 , Red)
      Call Lcd_fill_circle(80 , 30 , 15 , Blue)

      Call Lcd_fill_circle(22 , 80 , 10 , Blue)
      Call Lcd_fill_circle(138 , 80 , 10 , Green)
      Call Lcd_fill_circle(80 , 80 , 10 , Red)

      Call Lcd_fill_circle(22 , 110 , 5 , Red)
      Call Lcd_fill_circle(138 , 110 , 5 , Blue)
      Call Lcd_fill_circle(80 , 110 , 5 , Green)
      Call Lcd_text( "Circle Fill" , 40 , 50 , 2 , _
                      Black , Transparent)        'Text: sw; Circle
      Wait 3

'Darstellung Kreis
'~~~~~~~~~~~~~~~~~
      Call Lcd_clear(black)                       'Display: sw (lschen)
      Call Lcd_text( "Circle" , 55 , 1 , 2 , _
                      Yellow , Black)             'Text: ge; Circle
      Call Lcd_circle(80 , 68 , 50 , Green)
      Call Lcd_circle(80 , 68 , 40 , Red)
      Call Lcd_circle(80 , 68 , 30 , Blue)
      Call Lcd_circle(80 , 68 , 20 , Yellow)
      Call Lcd_circle(80 , 68 , 10 , Magenta)
      Call Lcd_circle(80 , 68 , 5 , Brown)
      Wait 3

'Darstellung Rechteck, gefllt
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      Call Lcd_clear(black)                       'Display: sw (lschen)
      Call Lcd_text( "Box Fill" , 45 , 2 , 2 , _
                      Yellow , Black)             'Text: ge; Box fill
      Call Lcd_box(20 , 20 , 120 , 88 , 1 , Yellow , Red)
      Call Lcd_box(30 , 30 , 100 , 68 , 1 , Green , Red)
      Call Lcd_box(40 , 40 , 80 , 48 , 1 , Red , Red)
      Call Lcd_box(50 , 50 , 60 , 28 , 1 , Blue , Red)
      Wait 3

'Darstellung Rechteck
'~~~~~~~~~~~~~~~~~~~~
      Call Lcd_clear(black)                       'Display: sw (lschen)
      Call Lcd_text( "Box " , 20 , 2 , 2 , _
                      Yellow , Black)             'Text: ge; Box
      Call Lcd_box(20 , 20 , 120 , 88 , 2 , Yellow , Red)
      Call Lcd_box(30 , 30 , 100 , 68 , 2 , Green , Blue)
      Call Lcd_box(40 , 40 , 80 , 48 , 2 , Red , Yellow)
      Call Lcd_box(50 , 50 , 60 , 28 , 2 , Blue , Green)
      Wait 3

      Call Lcd_clear(black)                       'Display: sw (lschen)
      Call Lcd_text( "Box" , 63 , 2 , 2 , _
                      Yellow , Black)             'Text: ge; Box
      Call Lcd_box(20 , 20 , 120 , 88 , 3 , Yellow , Red)
      Call Lcd_box(30 , 30 , 100 , 68 , 3 , Green , Blue)
      Call Lcd_box(40 , 40 , 80 , 48 , 3 , Red , Yellow)
      Call Lcd_box(50 , 50 , 60 , 28 , 3 , Blue , Green)
      Wait 3

'Darstellung Bilder (bei aktiver Speicherkarte)
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      #if Sdcard = 1
         Call Lcd_pic(0 , 0 , 160 , 128 , "bild01.bin")       'Artur
         Wait 5
         Call Lcd_pic(0 , 0 , 160 , 128 , "bild02.bin")       'Digedags
         Wait 5
         Call Lcd_pic(0 , 0 , 160 , 128 , "bild03.bin")       'Kaktusblte
         Wait 5
         Call Lcd_pic(0 , 0 , 160 , 128 , "bild04.bin")       'Sonnenblume
         Wait 5
         Call Lcd_clear(black)
      #endif
   #endif

M_ende:
'Ende Programmschleife
Loop

'Ende Programm
End

'-------------------------------------------------------------------------------
'40. === U N T E R P R O G R A M M =============================================
'-------------------------------------------------------------------------------
'Tonausgabe
'----------
A_410_upmor:                                      'Editmarker: UP10
'
Up_mor:
'Wandlung in ASCII
   Hs_mor = Ucase(hs_mor)                         'Zeichen gro
   Hm_mc = Asc(hs_mor)

   Select Case Hm_mc
      Case 0                                      'Pause 100 ms
         Waitms 100
         Goto Upmor_ret                           'Return
      Case Is < 58
         Hm_mc = Hm_mc - 48                       'Index ist nullbasiert
         Hs_mor = Lookupstr(hm_mc , Mc_zah)       'Morsecode Zahlen
      Case Else
         Hm_mc = Hm_mc - 65                       'Index ist nullbasiert
         Hs_mor = Lookupstr(hm_mc , Mc_buc)       'Morsecode Buchstaben
    End Select

   For Hz_mz = 1 To Len(hs_mor)
      Hs01 = Mid(hs_mor , Hz_mz , 1)

      If Hs01 = "." Then
         Set Ton : Waitms 70
      Else
         Set Ton : Waitms 210
      End If

      Reset Ton
      Waitms 70
   Next Hz_mz

Upmor_ret:
Return

'-------------------------------------------------------------------------------
'70. === I N C L U D E =========================================================
'-------------------------------------------------------------------------------
'
'Grafiktreiber (Funktionen)
   $include "TFTDriver\ST7735_functions.inc"

'SDCard (Initialisierung)
   #if Sdcard = 1
      $include "Avr-Dos\SD_card_init.inc"
   #endif

'Font (Schriftarten)
   $include "Font\Font8x8.font"
   $include "Font\Font12x16.font"
'  $include "Font\font6x10.font"                  'nicht in ST7735_functions
'  $include "Font\font8x12.font"                  'nicht in ST7735_functions
'  $include "Font\font10x16.font"                 'nicht in ST7735_functions

'-------------------------------------------------------------------------------
'80. === D A T E N B E R E I C H ===============================================
'-------------------------------------------------------------------------------
'
'Morsecode
'~~~~~~~~~
' Zahlen 0-9
Mc_zah:
   Data "-----" , ".----" , "..---" , "...--" , "....-" , "....." , "-...." , _
        "--..." , "---.." , "----."
' Buchstaben A-Z
Mc_buc:
   Data ".-" , "-..." , "-.-." , "-.." , "." , "..-." , "--." , "...." , _
        ".." , ".---" , "-.-" , ".-.." , "--" , ".-" , "---" , ".--." , _
        "--.-" , ".-." , "..." , "-" , "..-" , "...-" , ".--" , "-..-" , _
        "-.--" , "--.."

'Miky1:
'$bgf "miki.bgc"

'Ende Datenbereich

'###############################################################################

'Ende
